# -*- coding: utf-8 -*- #
# This file will be scheduled to run every one minute using cron to work
# on the waiting-list videos.
#
# Copyright (C) 2008, 2009 Leonardo Gastón De Luca leo[at]kde.org.ar
# Copyright (C) 2010 Osama Khalid osamak.wfm[at]gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import subprocess, tempfile, urllib2, os, sys
import settings, utilities, plugins
from django.core.management import setup_environ
setup_environ(settings)
from add.models import files
from datetime import datetime
from utilities import output

# To run on WebFaction's host.
sys.path = ['/home/osamak/webapps/django/', '/home/osamak/webapps/django/lib/python2.5'] + sys.path

class DowndloadLinkError(Exception):
  pass

def download(downloadURL, tempath):
    '''Downloads the video. Originally written by Leonardo'''
    output(downloadURL)
    remote_file = utilities.opener.open(downloadURL)#, timeout=30)
    local_file = open(tempath, 'wb')
    remote_size = int(remote_file.info()['content-length'])
    output("Downloading " + str(remote_size/1024) + "KB")
    remote_file_data = remote_file.read()
    local_file.write(remote_file_data)
    remote_file.close()
    local_file.close()

def ripAudio(tempath, cpath):
    '''Rips the audio in Ogg Vorbis'''
    devnull = '> /dev/null 2>&1'
    
    # Change this to your ffmpeg2theora location.
    subprocess.check_call(u'nice -n 19 /home/osamak/webapps/django/youogg/ffmpeg2theora "%s" --novideo -o "%s.oga" %s'
            % (tempath, cpath, devnull), shell = True)
    os.remove(tempath)

def convertVideo(tempath, cpath, isHD):
    '''Converts the video to Theora.'''
    devnull = '> /dev/null 2>&1'
    #if isHD: quality = "-v 8"
    #else: quality = ""

    # Change this to your ffmpeg2theora location.
    subprocess.check_call(u'nice -n 19 /home/osamak/webapps/django/youogg/ffmpeg2theora "%s" -o "%s.ogv" %s'
            % (tempath, cpath, devnull), shell = True)
    os.remove(tempath)

def main():
  for video in files.objects.filter(status='downloaded'): # There are downloaded vidoes, convert them.
    convertingFiles = files.objects.filter(status='converting')
    if len(convertingFiles): # There is a video being converted already, stop.
      output("There is a file that's already being converted, " + convertingFiles[0].videoID)
      break
    else:
      # You'll need to change the following to the files directory.
      cpath = os.path.join('/home/osamak/webapps/files', video.videoID)
      tempath = os.path.join(tempfile.gettempdir(), video.videoID)
      errorcounter = 0
      output("Converting " + video.videoID)
      video.status = 'converting'
      video.save()
      while errorcounter != 3:
        try:
          if video.localType == 'video':
            convertVideo(tempath, cpath, video.isHD)
          elif video.localType == 'audio':
            ripAudio(tempath, cpath)
          output(video.videoID + " finished.")
          video.status = 'done'
          video.save()
          break
        except subprocess.CalledProcessError, error:
          errorcounter += 1
          output("Something went wrong, retrying. %s. %s" % (errorcounter, error))
        except OSError: # The tmp file was removed by the server.
          pass
      if errorcounter == 3:
          output("Tried three times and faild. Give up!")
          video.status = 'failed'
          video.save()

  for video in files.objects.filter(status='queue'):
    # Here we have two error counters: one for "normal" erros and the other for HD.
    errorcounter = 0
    while True:
      try:
        if errorcounter > 3:
          output("Tried three times and faild. Give up!")
          raise DowndloadLinkError
        output(" ".join([video.localType, video.status, str(video.orgID), video.orgHost])) # REMOVE
        # Run the plugin to get the download link and the title.
        plugin = getattr(plugins, video.orgHost)(video.orgID)
        video.title = plugin.gettitle()
        downloadURL = plugin.getDownloadURL(video.isHD, errorcounter)
        if not downloadURL: # The video page URL is not correct, the plugin couldn't get the download URL right.
          raise DowndloadLinkError
        output("Downloading " + video.videoID)
        cpath = os.path.join('/home/osamak/webapps/files', video.videoID)
        tempath = os.path.join(tempfile.gettempdir(), video.videoID)
        video.status = 'downloading'
        video.save()
        download(downloadURL, tempath)
        video.status = 'downloaded'
        video.save()
        return # One file per loop. This will prevent repeated downloads.

      except (KeyboardInterrupt, DowndloadLinkError):
        video.status = 'failed'
        video.save()
        output("Something went wrong, skipping.")
      except (urllib2.URLError, urllib2.HTTPError, urllib2.socket.timeout), error:
        errorcounter += 1
        output("Something went wrong, retrying. %s. %s" % (errorcounter, error))
        continue
      #except (urllib2.socket.timeout):
      #  pass
      break

if __name__ == "__main__":
    main()
